import random

from testv.models.enums.cap_type import CapTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.cap_amount import CapAmountRule
from tests.unit.abstract_test_case import CommonTestCase


class CapAmountTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        only valid attributes can be validated
        """

        with self.assertRaises(AssertionError):
            CapAmountRule(cap_type=None, cap_automatic=self._faker.pybool(),
                          cap_amount=float(self._faker.random_int(min=1)), cap_fields={})

        try:
            CapAmountRule(cap_type=CapTypeEnum.EVEN,
                          cap_automatic=self._faker.pybool(),
                          cap_amount=float(self._faker.random_int(min=1)),
                          cap_fields={})
        except Exception:
            self.fail("Unexpected exception found")

    def test_execute_validation(self):
        """
        case
            1) must be undefined if "t1_spend_cap_type" == "no-limit". This is because no-limit overrides this setting in T1
            2) must be undefined if "t1_spend_cap_automatic == TRUE OR undefined, because it is meaningless otherwise
            3) required if "t1_spend_cap_automatic == FALSE
            4) must be undefined if t1_spend_cap_type is undefined too
        """

        # must be undefined if "t1_spend_cap_type" == "no-limit". This is because no-limit overrides this setting in T1
        with self.assertRaises(ValidationException):
            CapAmountRule(cap_type=CapTypeEnum.NO_LIMIT,
                          cap_automatic=self._faker.pybool(),
                          cap_amount=float(self._faker.random_int(min=1)),
                          cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()

        # must be undefined if "t1_spend_cap_automatic == TRUE OR undefined, because it is meaningless otherwise
        with self.assertRaises(ValidationException):
            CapAmountRule(
                cap_type=random.choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                cap_automatic=True,
                cap_amount=float(self._faker.random_int(min=1)),
                cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()

        # required if "t1_spend_cap_automatic == FALSE
        with self.assertRaises(ValidationException):
            CapAmountRule(
                cap_type=random.choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                cap_automatic=False,
                cap_amount=None,
                cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()

        try:
            CapAmountRule(
                cap_type=random.choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                cap_automatic=True,
                cap_amount=None,
                cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()
        except Exception:
            self.fail("Unexpected exception")
