import random

from testv.models.enums.cap_type import CapTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.cap_amount_put import CapAmountPutRule
from tests.unit.abstract_test_case import CommonTestCase


class CapAmountPutTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        only valid attributes can be validated
        """
        with self.assertRaises(AssertionError):
            CapAmountPutRule(cap_type=CapTypeEnum.set(None),
                             cap_automatic=self._faker.pybool(),
                             cap_amount=self._faker.pystr(),
                             cap_fields={})

        try:
            CapAmountPutRule(cap_type=CapTypeEnum.set(None),
                             cap_automatic=self._faker.pybool(),
                             cap_amount=self._faker.pyfloat(positive=True),
                             cap_fields={})
        except Exception:
            self.fail("Unexpected exception found")

    def test_execute_validation(self):
        """
        case
            1) it is OK only when the DB's t1_spend_cap_type is asap or even AND the DB's
            t1_spend_cap_automatic is FALSE
            2) if DB t1_spend_cap_type is asap or even, but current t1_spend_cap_automatic is TRUE,
                then t1_spend_cap_automatic is also required, and it must be FALSE.
            3) if DB t1_spend_cap_type is no-limit, then t1_spend_cap_type is required and must be asap or even,
                and t1_spend_cap_automatic is also required and must be FALSE.
        """

        # it is OK only when the DB's t1_spend_cap_type is asap or even AND the DB's t1_spend_cap_automatic is FALSE
        with self.assertRaises(ValidationException):
            CapAmountPutRule(cap_type=random.choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                             cap_automatic=True,
                             cap_amount=self._faker.pyfloat(positive=True),
                             cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()

        # if DB t1_spend_cap_type is asap or even, but current t1_spend_cap_automatic is TRUE,
        # then t1_spend_cap_automatic is also required, and it must be FALSE.
        with self.assertRaises(ValidationException):
            CapAmountPutRule(cap_type=random.choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                             cap_automatic=True,
                             cap_amount=self._faker.pyfloat(positive=True),
                             cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()

        # if DB t1_spend_cap_type is no-limit, then t1_spend_cap_type is required and must be asap or even,
        # and t1_spend_cap_automatic is also required and must be FALSE.
        with self.assertRaises(ValidationException):
            CapAmountPutRule(cap_type=CapTypeEnum.NO_LIMIT,
                             cap_automatic=True,
                             cap_amount=self._faker.pyfloat(positive=True),
                             cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()

        try:
            CapAmountPutRule(cap_type=random.choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                             cap_automatic=False,
                             cap_amount=self._faker.pyfloat(positive=True),
                             cap_fields={"cap_automatic": "", "cap_type": "", "cap_amount": ""}).execute()
        except Exception:
            self.fail("Unexpected exception")
