import random

from testv.models.enums.cap_type import CapTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.cap_automatic import CapAutomaticRule
from tests.unit.abstract_test_case import CommonTestCase


class CapAutomaticTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        only valid attributes can be validated
        """

        with self.assertRaises(AssertionError):
            CapAutomaticRule(None, self._faker.pybool()).execute()

        try:
            CapAutomaticRule(CapTypeEnum.EVEN, self._faker.pybool()).execute()
        except Exception:
            self.fail("Unexpected exception found")

    def test_execute_validation(self):
        """
        case
            1) must be undefined if "t1_spend_cap_type" == "no-limit". This is because no-limit overrides this setting in T1
            2) required if "t1_spend_cap_type" != "no-limit", but is defined
            3) must be undefined if t1_spend_cap_type is undefined too
        """

        # must be undefined if "t1_spend_cap_type" == "no-limit". This is because no-limit overrides this setting in T1
        with self.assertRaises(ValidationException):
            CapAutomaticRule(CapTypeEnum.NO_LIMIT,
                             self._faker.pybool(), {"cap_automatic": "", "cap_type": ""}).execute()

        # required if "t1_spend_cap_type" != "no-limit", but is defined
        with self.assertRaises(ValidationException):
            CapAutomaticRule(random.choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                             None, {"cap_automatic": "", "cap_type": ""}).execute()

        # must be undefined if t1_spend_cap_type is undefined too
        with self.assertRaises(ValidationException):
            CapAutomaticRule(CapTypeEnum.UNDEFINED,
                             self._faker.pybool(), {"cap_automatic": "", "cap_type": ""}).execute()
