from testv.models.enums.cap_type import CapTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.cap_automatic_put import CapAutomaticPutRule
from tests.unit.abstract_test_case import CommonTestCase


class CapAutomaticPutTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        only valid attributes can be validated
        """

        with self.assertRaises(AssertionError):
            CapAutomaticPutRule(None, self._faker.pybool(), self._faker.pyfloat())

        with self.assertRaises(AssertionError):
            CapAutomaticPutRule(CapTypeEnum.EVEN, self._faker.pystr(), None)

        try:
            CapAutomaticPutRule(CapTypeEnum.EVEN, self._faker.pybool(), self._faker.pyfloat())
        except Exception:
            self.fail("Unexpected exception found")

    def test_execute_validation(self):
        """
        case
            1) it can defined alone as TRUE, only if t1_spend_cap_type in DB is asap or even.
            2) it can be defined alone as FALSE only if  DB's t1_spend_cap_amount is already defined.
        """

        # it can defined alone as TRUE, only if t1_spend_cap_type in DB is asap or even.
        with self.assertRaises(ValidationException):
            CapAutomaticPutRule(CapTypeEnum.NO_LIMIT, True, None, {
                "cap_automatic": "",
                "cap_type": ""
            }).execute()

        # it can be defined alone as FALSE only if  DB's t1_spend_cap_amount is already defined.
        with self.assertRaises(ValidationException):
            CapAutomaticPutRule(CapTypeEnum.UNDEFINED, False, None, {
                "cap_automatic": "",
                "cap_type": ""
            }).execute()
