from testv.models.enums.cap_type import CapTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.cap_type_undefined import CapTypeUndefinedRule
from tests.unit.abstract_test_case import CommonTestCase


class CapTypeUndefinedTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        only valid attributes can be validated
        """

        with self.assertRaises(AssertionError):
            CapTypeUndefinedRule(None, CapTypeEnum.EVEN).execute()

        with self.assertRaises(AssertionError):
            CapTypeUndefinedRule(CapTypeEnum.EVEN, None).execute()

        try:
            CapTypeUndefinedRule(CapTypeEnum.EVEN, CapTypeEnum.EVEN).execute()
        except Exception:
            self.fail("Unexpected exception found")

    def test_execute_validation(self):
        """
        case
        required if t1_impression_cap_type is undefined (one or the other must be defined at a minimum)
        """

        # both arguments are undefined, we expecting an Exception
        with self.assertRaises(ValidationException):
            CapTypeUndefinedRule(CapTypeEnum.UNDEFINED, CapTypeEnum.UNDEFINED).execute()

        # first argument is undefined, but second is OK
        try:
            CapTypeUndefinedRule(CapTypeEnum.UNDEFINED, CapTypeEnum.EVEN).execute()
        except Exception:
            self.fail("Unexpected exception found")

        # second argument is undefined, but first is OK
        try:
            CapTypeUndefinedRule(CapTypeEnum.EVEN, CapTypeEnum.UNDEFINED).execute()
        except Exception:
            self.fail("Unexpected exception found")
