from datetime import timedelta

from testv.validators.exceptions import ValidationException
from testv.validators.rules.dates_overlap import DatesOverlapRule
from tests.unit.abstract_test_case import CommonTestCase


class DatesOverlapTestCase(CommonTestCase):
    def setUp(self):
        self.date_format = "%Y-%m-%dT%H:%M"

    def test_execute_validation(self):
        """
        case:
        start_date is later than end_date. Message "start_date must be earlier than end_date"
        start_time is later than end_time. Message " start_time must be earlier than end_time"
        start_date is in the past. Message "start_date must be in the future"
        """
        # invalid zone name
        with self.assertRaises(ValidationException):
            start_date = self._faker.date_time_between(start_date="+2d", end_date="+3d")
            DatesOverlapRule(start_date=start_date.strftime(self.date_format),
                             end_date=(start_date + timedelta(minutes=1)).strftime(self.date_format),
                             zone_name=self._faker.pystr()).execute()

        # start_date must be in future
        with self.assertRaises(ValidationException):
            DatesOverlapRule(start_date=self._faker.date_time_between(start_date="-4d",
                                                                      end_date="-1d").strftime(self.date_format),
                             end_date=self._faker.date_time_between(start_date="now",
                                                                    end_date="+1d").strftime(self.date_format),
                             zone_name=self._faker.timezone()).execute()

        # start_date is later than end_date. Message "start_date must be earlier than end_date"
        with self.assertRaises(ValidationException):
            DatesOverlapRule(start_date=self._faker.date_time_between(start_date="+2d",
                                                                      end_date="+3d").strftime(self.date_format),
                             end_date=self._faker.date_time_between(start_date="now",
                                                                    end_date="+1d").strftime(self.date_format),
                             zone_name=self._faker.timezone()).execute()

        # start_time is the same to end_time. Message " start_time must be earlier than end_time"
        with self.assertRaises(ValidationException):
            start_date = self._faker.date_time_between(start_date="+2d", end_date="+3d")
            DatesOverlapRule(start_date=start_date.strftime(self.date_format),
                             end_date=start_date.strftime(self.date_format),
                             zone_name=self._faker.timezone()).execute()

        # start_time is later than end_time. Message " start_time must be earlier than end_time"
        with self.assertRaises(ValidationException):
            start_date = self._faker.date_time_between(start_date="+2d", end_date="+3d")
            DatesOverlapRule(start_date=start_date.strftime(self.date_format),
                             end_date=(start_date - timedelta(minutes=1)).strftime(self.date_format),
                             zone_name=self._faker.timezone()).execute()

        try:
            start_date = self._faker.date_time_between(start_date="+2d", end_date="+3d")
            DatesOverlapRule(start_date=start_date.strftime(self.date_format),
                             end_date=(start_date + timedelta(minutes=1)).strftime(self.date_format),
                             zone_name=self._faker.timezone()).execute()
        except Exception:
            self.fail("Unexpected exception")
