from testv.models.enums.frequency_interval import FrequencyIntervalEnum
from testv.models.enums.frequency_type import FrequencyTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.frequency_required_fields import FrequencyRequiredFieldsRule
from tests.unit.abstract_test_case import CommonTestCase


class FrequencyRequiredFieldsRuleTestCase(CommonTestCase):
    def test_frequency_required_fields_rules(self):
        """
            rules for 't1_frequency_type':
            required if frequency_type is FrequencyTypeEnum.UNDEFINED and frequency_interval is not FrequencyIntervalEnum.UNDEFINED
                - error
            must be undefined if t1_frequency_optimization is TRUE
                - error message: "{:field_name} cannot be defined when t1_frequency_optimization is TRUE"
        """

        with self.assertRaises(ValidationException) as e:
            FrequencyRequiredFieldsRule(frequency_type=FrequencyTypeEnum.UNDEFINED, frequency_optimization=None,
                                        frequency_interval=FrequencyIntervalEnum.DAILY, frequency_amount=None,
                                        cap_fields={"frequency_optimization": "t1_frequency_optimization",
                                                    "frequency_type": "t1_frequency_type",
                                                    "frequency_amount": "t1_frequency_amount",
                                                    "frequency_interval": "t1_frequency_interval"}).execute()

        self.assertRegex(str(e.exception), "(?i)^if you define t1_frequency_interval")

        with self.assertRaises(ValidationException) as e:
            FrequencyRequiredFieldsRule(frequency_type=FrequencyTypeEnum.UNDEFINED, frequency_optimization=None,
                                        frequency_interval=FrequencyIntervalEnum.UNDEFINED, frequency_amount=self._faker.pyfloat(positive=True),
                                        cap_fields={"frequency_optimization": "t1_frequency_optimization",
                                                    "frequency_type": "t1_frequency_type",
                                                    "frequency_amount": "t1_frequency_amount",
                                                    "frequency_interval": "t1_frequency_interval"}).execute()

        self.assertRegex(str(e.exception), "(?i)^if you define t1_frequency_amount")
