from testv.models.enums.frequency_interval import FrequencyIntervalEnum
from testv.models.enums.frequency_type import FrequencyTypeEnum
from testv.models.enums.input_type import InputTypeEnum
from testv.validators.rules.frequency_type_put import FrequencyTypePutRule
from tests.unit.abstract_test_case import CommonTestCase


class FrequencyTypePutRuleTestCase(CommonTestCase):
    def test_put_reset_amount_to_int_for_fixed(self):
        """
        case: when input type switching to 'fixed', amount should be transformed into int
        """
        expecting_value = self._faker.pyint()

        try:
            rule = FrequencyTypePutRule(frequency_optimization=None,
                                        frequency_type=FrequencyTypeEnum.UNDEFINED,
                                        frequency_amount=float(expecting_value),
                                        frequency_interval=FrequencyIntervalEnum.DAILY,
                                        input_type=InputTypeEnum.FIXED)

            item = rule.put(FrequencyTypeEnum.ASAP, FrequencyIntervalEnum.DAILY, frequency_amount=float(expecting_value))
            self.assertEqual(expecting_value, item.frequency_amount)
        except BaseException as e:
            self.fail(str(e))

    def test_put_reset_amount_to_int_should_work_with_none(self):
        """
        case: when input type switching to 'fixed', int transformation won't happen for None value
        """
        try:
            rule = FrequencyTypePutRule(frequency_optimization=False,
                                        frequency_type=FrequencyTypeEnum.ASAP,
                                        frequency_amount=None,
                                        frequency_interval=FrequencyIntervalEnum.DAILY,
                                        input_type=InputTypeEnum.FIXED)

            rule.put(frequency_type=None, frequency_interval=None, frequency_amount=None)
        except BaseException as e:
            self.fail(str(e))

    def test_frequency_type_frequency_optimization_reset_fields(self):
        """
        case:
        while updating fields, frequency frequency_optimization of no should set rest of the fields:
        amount and interval
        """
        rule = FrequencyTypePutRule(frequency_optimization=True,
                                    frequency_type=FrequencyTypeEnum.UNDEFINED,
                                    frequency_amount=self._faker.pyfloat(),
                                    frequency_interval=FrequencyIntervalEnum.DAILY,
                                    input_type=InputTypeEnum.FIXED)

        expecting_amount = self._faker.pyfloat()
        expecting_interval = FrequencyIntervalEnum.HOUR

        rule.put(frequency_type=FrequencyTypeEnum.ASAP,
                 frequency_interval=expecting_interval,
                 frequency_amount=expecting_amount)

        self.assertEqual(expecting_amount, rule.frequency_amount)
        self.assertEqual(FrequencyTypeEnum.ASAP, rule.frequency_type)
        self.assertEqual(expecting_interval, rule.frequency_interval)

    def test_frequency_type_no_limit_reset_fields(self):
        """
        case:
        while updating fields, frequency frequency_type of no-limit should set rest of the fields:
        amount and interval
        """
        rule = FrequencyTypePutRule(frequency_optimization=False,
                                    frequency_type=FrequencyTypeEnum.NO_LIMIT,
                                    frequency_amount=self._faker.pyfloat(),
                                    frequency_interval=FrequencyIntervalEnum.DAILY,
                                    input_type=InputTypeEnum.FIXED)

        expecting_amount = self._faker.pyfloat()
        expecting_interval = FrequencyIntervalEnum.HOUR

        rule.put(frequency_type=FrequencyTypeEnum.NO_LIMIT,
                 frequency_interval=expecting_interval,
                 frequency_amount=expecting_amount)

        self.assertEqual(expecting_amount, rule.frequency_amount)
        self.assertEqual(expecting_interval, rule.frequency_interval)
