from testv.validators.exceptions import ValidationException
from testv.validators.rules.impression_budget_rule import ImpressionBudgetRule
from tests.unit.abstract_test_case import CommonTestCase


class ImpressionBudgetRuleTestCase(CommonTestCase):
    def test_wrong_type_impression_cap_automatic_failed(self):
        """
        case: impression_cap_automatic should be bool
        """

        with self.assertRaises(AssertionError) as e:
            ImpressionBudgetRule(impression_cap_automatic=self._faker.pystr(),
                                 total_impression_budget=None)

        self.assertRegex(str(e.exception), "^impression_cap_automatic should be bool")

    def test_invalid_type_total_impression_budget(self):
        """
        case: total_impression_budget should be int or None
        """

        with self.assertRaises(AssertionError) as e:
            ImpressionBudgetRule(impression_cap_automatic=self._faker.pybool(),
                                 total_impression_budget=self._faker.pystr())

        self.assertRegex(str(e.exception), "^total_impression_budget should be")

    def test_assertion_raises_for_none_budget(self):
        """
        case: for impression_cap_automatic True, budget should be more or equal to 1
        """

        with self.assertRaises(ValidationException) as e:
            ImpressionBudgetRule(impression_cap_automatic=True,
                                 total_impression_budget=None).execute()

        self.assertRegex(str(e.exception), "^Cannot set automatic daily impression")

    def test_assertion_passed_for_positive_budget(self):
        """
        case: for impression_cap_automatic True, budget should be more or equal to 1
        """

        try:
            ImpressionBudgetRule(impression_cap_automatic=True,
                                 total_impression_budget=self._faker.random_int(min=1)).execute()
        except Exception as e:
            raise(str(e))
