from random import choice

from testv.models.enums.cap_type import CapTypeEnum
from testv.models.enums.input_type import InputTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.impression_cap_type import ImpressionCapTypeRule
from tests.unit.abstract_test_case import CommonTestCase


class ImpressionCapTypeTestCase(CommonTestCase):
    def test_amount_has_wrong_type(self):
        """
        rule should trigger an exception for a wrong type of amount attribute
        :return:
        """
        with self.assertRaises(AssertionError) as e:
            ImpressionCapTypeRule(impression_cap_type=CapTypeEnum.UNDEFINED,
                                  impression_cap_automatic=self._faker.pybool(),
                                  impression_cap_amount=self._faker.pystr(),
                                  input_type=InputTypeEnum.UNDEFINED)

        self.assertRegex(str(e.exception), "^impression_cap_amount should be")

    def test_validation_ignored_for_undefined_impression(self):
        """
        rule should be ignored for undefined impression
        :return:
        """
        try:
            ImpressionCapTypeRule(impression_cap_type=CapTypeEnum.UNDEFINED,
                                  impression_cap_automatic=self._faker.pybool(),
                                  impression_cap_amount=self._faker.pyint(),
                                  input_type=InputTypeEnum.UNDEFINED).execute()
        except BaseException as e:
            self.fail(str(e))

    def test_validation_ignored_for_no_limit_type(self):
        """
        rule should be ignored for no-limit type
        :return:
        """
        try:
            ImpressionCapTypeRule(impression_cap_type=CapTypeEnum.NO_LIMIT,
                                  impression_cap_automatic=self._faker.pybool(),
                                  impression_cap_amount=self._faker.pyint(),
                                  input_type=InputTypeEnum.UNDEFINED).execute()
        except BaseException as e:
            self.fail(str(e))

    def test_exception_raises_for_invalid_type_for_fixed(self):
        """
        rule should trigger an exception for a wrong type of amount attribute,

        for fixed input type
        :return:
        """
        with self.assertRaises(ValidationException) as e:
            ImpressionCapTypeRule(impression_cap_type=choice([CapTypeEnum.ASAP, CapTypeEnum.EVEN]),
                                  impression_cap_automatic=False,
                                  impression_cap_amount=self._faker.pyfloat(right_digits=2),
                                  input_type=InputTypeEnum.FIXED).execute()

        self.assertRegex(str(e.exception), "^for input_type='fixed' impression_cap_amount")
