from random import choice

from testv.models.enums.input_type import InputTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.common_required_fields_post import CommonRequiredFieldsPostRule
from tests.unit.abstract_test_case import CommonTestCase


class InputTypeRequiredRuleTestCase(CommonTestCase):
    def test_wrong_type_input_type_failed(self):
        """
        case: input_type should be type of InputTypeEnum
        """

        with self.assertRaises(AssertionError):
            CommonRequiredFieldsPostRule(input_type=self._faker.pystr())

    def test_undefined_input_type_failed(self):
        """
        case: input_type is required field
        """

        with self.assertRaises(ValidationException) as e:
            CommonRequiredFieldsPostRule(input_type=InputTypeEnum.set(self._faker.pystr())).execute()

        self.assertRegex(str(e.exception), "^input_type value must be one of")

    def test_input_type_valid(self):
        """
        case: input_type doesn't trigger an exception for valid value
        """

        try:
            CommonRequiredFieldsPostRule(input_type=choice([InputTypeEnum.PERCENTAGE, InputTypeEnum.FIXED])).execute()
        except BaseException:
            self.fail("Unexpected exception")
