from testv.models.enums.pacing_interval import PacingIntervalEnum
from testv.models.enums.pacing_type import PacingTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.pacing_type import PacingTypeRule
from tests.unit.abstract_test_case import CommonTestCase


class PacingTypeRuleTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        pacing_type and pacing_interval should have valid type

        """

        with self.assertRaises(AssertionError):
            PacingTypeRule(pacing_type=None,
                           pacing_amount=None,
                           pacing_interval=PacingTypeEnum.set(None))

        with self.assertRaises(AssertionError):
            PacingTypeRule(pacing_type=PacingTypeEnum.set(None),
                           pacing_amount=None,
                           pacing_interval=None)

        try:
            PacingTypeRule(pacing_type=PacingTypeEnum.set(None),
                           pacing_amount=None,
                           pacing_interval=PacingIntervalEnum.set(None))
        except AssertionError:
            self.fail("Unexpected exception")

    def test_pacing_type_is_defined(self):
        """
        case
        pacing_amount required if pacing_type is defined
        """

        with self.assertRaises(ValidationException):
            PacingTypeRule(pacing_type=PacingTypeEnum.ASAP,
                           pacing_amount=None,
                           pacing_interval=PacingIntervalEnum.set(None)).execute()

    def test_pacing_interval_is_defined(self):
        """
        case
        pacing_interval required if pacing_type is defined
        """

        with self.assertRaises(ValidationException) as e:
            PacingTypeRule(pacing_type=PacingTypeEnum.ASAP,
                           pacing_amount=self._faker.pyfloat(),
                           pacing_interval=PacingIntervalEnum.set(None)).execute()

        self.assertRegex(str(e.exception), "(?i)^pacing_interval is required")

    def test_pacing_type_is_required_when_interval_provided(self):
        """
        case
        pacing_type is required if pacing_interval is provided
        """
        with self.assertRaises(ValidationException) as e:
            PacingTypeRule(pacing_type=PacingTypeEnum.UNDEFINED,
                           pacing_amount=self._faker.pyfloat(),
                           pacing_interval=PacingIntervalEnum.DAY).execute()

        self.assertRegex(str(e.exception), "(?i)^pacing_type is required")

    def test_pacing_interval_is_required_when_amount_provided(self):
        """
        case
        pacing_interval is required if pacing_amount is provided
        """
        with self.assertRaises(ValidationException) as e:
            PacingTypeRule(pacing_type=PacingTypeEnum.UNDEFINED,
                           pacing_amount=self._faker.pyfloat(),
                           pacing_interval=PacingIntervalEnum.UNDEFINED).execute()

        self.assertRegex(str(e.exception), "(?i)^pacing_interval is required")

    def test_execute_valid(self):
        """
        case
        no exceptions for valid data
        :return:
        """

        try:
            PacingTypeRule(pacing_type=PacingTypeEnum.ASAP,
                           pacing_amount=self._faker.pyfloat(),
                           pacing_interval=PacingIntervalEnum.DAY)
        except Exception:
            self.fail("Unexpected exception")
