from testv.models.enums.dimension_action import DimensionActionEnum
from testv.models.enums.dimension_code import DimensionCodeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.strategy_dimension_required_fields_post import \
    StrategyDimensionPostRequiredFieldsRule
from tests.unit.abstract_test_case import CommonTestCase


class StrategyDimensionPostRequiredFieldsRuleTestCase(CommonTestCase):
    def test_init_assertion_failed(self):
        """
        case:
        - dimension_code should be DimensionCodeEnum
        - action should be DimensionActionEnum
        """

        # dimension_code should be DimensionCodeEnum
        with self.assertRaises(AssertionError):
            StrategyDimensionPostRequiredFieldsRule(dimension_code=None,
                                                    action=DimensionActionEnum.UNDEFINED,
                                                    include_value_ids=self._faker.pystr(),
                                                    exclude_value_ids=self._faker.pystr())

        # action should be DimensionActionEnum
        with self.assertRaises(AssertionError):
            StrategyDimensionPostRequiredFieldsRule(dimension_code=DimensionCodeEnum.UNDEFINED,
                                                    action=None,
                                                    include_value_ids=self._faker.pystr(),
                                                    exclude_value_ids=self._faker.pystr())

    def test_init_assertion_pass(self):
        """
        case:
        - object init properly with correct data
        """

        try:
            StrategyDimensionPostRequiredFieldsRule(dimension_code=DimensionCodeEnum.UNDEFINED,
                                                    action=DimensionActionEnum.UNDEFINED,
                                                    include_value_ids=self._faker.pystr(),
                                                    exclude_value_ids=self._faker.pystr())
        except Exception:
            self.fail("Unexpected exception")

    def test_dimension_code_required(self):
        """
        case
        - dimension_code is required
        """

        with self.assertRaises(ValidationException) as cm:
            StrategyDimensionPostRequiredFieldsRule(dimension_code=DimensionCodeEnum.UNDEFINED,
                                                    action=DimensionActionEnum.UNDEFINED,
                                                    include_value_ids=self._faker.pystr(),
                                                    exclude_value_ids=self._faker.pystr()).execute()

        self.assertRegex(str(cm.exception), "dimension_code")

    def test_action_required(self):
        """
        case
        - action is required
        """

        with self.assertRaises(ValidationException) as cm:
            StrategyDimensionPostRequiredFieldsRule(dimension_code=DimensionCodeEnum.DMAX,
                                                    action=DimensionActionEnum.UNDEFINED,
                                                    include_value_ids=self._faker.pystr(),
                                                    exclude_value_ids=self._faker.pystr()).execute()

        self.assertRegex(str(cm.exception), "action")

    def test_one_of_value_ids_required(self):
        """
        case
        - at least one of include_value_ids or exclude_value_ids must be defined
        """

        with self.assertRaises(ValidationException) as cm:
            StrategyDimensionPostRequiredFieldsRule(dimension_code=DimensionCodeEnum.DMAX,
                                                    action=DimensionActionEnum.OVERWRITE,
                                                    include_value_ids=None,
                                                    exclude_value_ids=None).execute()

        self.assertRegex(str(cm.exception), "include_value_ids or exclude_value_ids")

    def test_execution_valid(self):
        """
        case:
        nothing triggered for valid parameters
        """

        try:
            StrategyDimensionPostRequiredFieldsRule(dimension_code=DimensionCodeEnum.DMAX,
                                                    action=DimensionActionEnum.OVERWRITE,
                                                    include_value_ids=self._faker.pystr(),
                                                    exclude_value_ids=self._faker.pystr()).execute()
        except Exception:
            self.fail("Unexpected exception")
