from testv.models.enums.impression_pacing_type import ImpressionPacingTypeEnum
from testv.models.enums.pacing_type import PacingTypeEnum
from testv.validators.exceptions import ValidationException
from testv.validators.rules.strategy_required_fields_post import StrategyPostRequiredFieldsRule
from tests.unit.abstract_test_case import CommonTestCase


class StrategyPostRequiredFieldsRuleTestCase(CommonTestCase):
    def test_execute_assert_failed(self):
        """
        case:
        at least one of the following is required: pacing_type, impression_pacing_type,
        min_bid, max_bid, goal_value, roi_target, frequency_optimization

        """

        with self.assertRaises(ValidationException):
            StrategyPostRequiredFieldsRule(pacing_type=PacingTypeEnum.set(None),
                                           impression_pacing_type=ImpressionPacingTypeEnum.set(None),
                                           min_bid=None, max_bid=None,
                                           goal_value=None,
                                           roi_target=None,
                                           frequency_optimization=None).execute()

    def test_execute_validation(self):
        """
        case
        when at least 1 of parameters is not None validation should pass without exceptions
        """

        # both arguments are undefined, we expecting an Exception
        try:
            StrategyPostRequiredFieldsRule(pacing_type=PacingTypeEnum.set(None),
                                           impression_pacing_type=ImpressionPacingTypeEnum.set(None),
                                           min_bid=self._faker.pyfloat(), max_bid=None,
                                           goal_value=None, roi_target=None,
                                           frequency_optimization=None).execute()
        except ValidationException:
            self.fail("Unexpected exception")
