from mock.mock import Mock

from testv.validators.exceptions import ValidationException
from testv.validators.rules.t1_advertiser import T1AdvertiserRule
from tests.unit.abstract_test_case import CommonTestCase


class T1AdvertiserRuleTestCase(CommonTestCase):
    def setUp(self):
        t1_mock = Mock(spec=["getCurrencyRate", "isValidAdvertiser"])
        t1_mock.getCurrencyRate.return_value = 1.1
        t1_mock.isValidAdvertiser.return_value = True

        self.t1_service = t1_mock

    def test_advertiser_has_invalid_type(self):
        """
        case:
        nothing triggered for valid parameters
        """
        with self.assertRaises(ValidationException) as cm:
            T1AdvertiserRule(advertiser_id=-1,
                             t1_service=self.t1_service).execute()

            self.assertRegex(str(cm.exception), "^invalid advertiser_id")

        with self.assertRaises(ValidationException) as cm:
            T1AdvertiserRule(advertiser_id="a",
                             t1_service=self.t1_service).execute()

            self.assertRegex(str(cm.exception), "^invalid advertiser_id")

        with self.assertRaises(ValidationException) as cm:
            T1AdvertiserRule(advertiser_id=None,
                             t1_service=self.t1_service).execute()

            self.assertRegex(str(cm.exception), "^invalid advertiser_id")

    def test_advertiser_not_found(self):
        """
        case:
        nothing triggered for valid parameters
        """
        with self.assertRaises(ValidationException) as cm:
            self.t1_service.isValidAdvertiser.return_value = False
            T1AdvertiserRule(advertiser_id=self._faker.pyint(min_value=1),
                             t1_service=self.t1_service).execute()

            self.assertRegex(str(cm.exception), "^invalid advertiser_id")

    def test_rule_works_fine(self):
        """
        case:
        nothing triggered for valid parameters
        """
        try:
            T1AdvertiserRule(advertiser_id=self._faker.pyint(min_value=1),
                             t1_service=self.t1_service).execute()

        except BaseException as e:
            self.fail(str(e))
