from testv.validators.rules.t1_value_ids import T1ValueIdsRule
from tests.unit.abstract_test_case import CommonTestCase


class T1ValueIdsRuleTestCase(CommonTestCase):
    def test_init_assertion_failed(self):
        """
        case:
        - value_ids should be a str or None
        - field_name should be a str
        """

        # value_ids should be a str or None
        with self.assertRaises(AssertionError):
            T1ValueIdsRule(value_ids=self._faker.pyint(),
                           field_name=self._faker.pystr())

        # field_name should be a str
        with self.assertRaises(AssertionError):
            T1ValueIdsRule(value_ids=None,
                           field_name=None)

    def test_value_ids_can_be_null(self):
        """
        case:
        - ignore whole rule if value_ids is None
        """

        try:
            T1ValueIdsRule(value_ids=None,
                           field_name=self._faker.pystr()).execute()
        except Exception:
            self.fail("Unexpected exception")

    def test_value_ids_should_have_valid_format(self):
        """
        case:
        - format of provided "ids" is something like: num,num,num
        """

        with self.assertRaises(AssertionError) as cm:
            T1ValueIdsRule(value_ids=self._faker.pystr(),
                           field_name=self._faker.pystr()).execute()

        self.assertRegex(str(cm.exception), "value_ids should be a")

    def test_rule_works_fine(self):
        """
        case:
        nothing triggered for valid parameters
        """

        try:
            T1ValueIdsRule(value_ids=["1", "2", "3"],
                           field_name=self._faker.pystr()).execute()
        except Exception:
            self.fail("Unexpected exception")
